package org.mycompany.filedownloadtest;

import com.vaadin.annotations.Theme;
import com.vaadin.annotations.Widgetset;
import com.vaadin.cdi.CDIUI;
import com.vaadin.server.FileDownloader;
import com.vaadin.server.Page;
import com.vaadin.server.StreamResource;
import com.vaadin.server.VaadinRequest;
import com.vaadin.ui.Button;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.Label;
import com.vaadin.ui.UI;
import com.vaadin.ui.VerticalLayout;
import java.io.ByteArrayInputStream;
import javax.enterprise.inject.Instance;
import javax.inject.Inject;

@CDIUI("")
@Theme("mytheme")
@Widgetset("org.mycompany.filedownloadtest.MyAppWidgetset")
public class MyUI extends UI {

    private static final long serialVersionUID = 1L;
    public static final String TEXT_TO_DOWNLOAD = "Lorem ipsum dolor sit amet, consectetur adipiscing elit. "
            + "Ut id dui quis arcu ultrices fringilla ac vel lorem. "
            + "Nulla risus turpis, posuere quis aliquam sit amet, consequat ut dolor. "
            + "Pellentesque iaculis dapibus mollis. Proin nec velit in purus placerat aliquam. "
            + "Quisque fringilla tincidunt lorem vitae porttitor. "
            + "Morbi efficitur nunc sit amet arcu venenatis, nec venenatis libero commodo. "
            + "Etiam facilisis sit amet nulla eu dignissim. Proin ut eros enim. "
            + "Pellentesque ac nisl sit amet lorem faucibus laoreet. "
            + "In et ante in lacus dapibus tincidunt ac ac diam. "
            + "Donec efficitur efficitur elit id posuere.";

    @Inject
    @Download
    private Instance<DownloadTextWindow> downloadTextWindow;

    @Override
    protected void init(VaadinRequest vaadinRequest) {
        final VerticalLayout layout = new VerticalLayout();

        final HorizontalLayout buttonLayout = new HorizontalLayout();
        buttonLayout.setWidth("100%");

        final Button downloadText = new Button("Download w/o window vaadin7 style");
        downloadText.setImmediate(true);
        downloadText.addClickListener(e -> {
            download(downloadText, true);
        });
        final Button downloadTextOldStyle = new Button("Download w/o window old style");
        downloadTextOldStyle.addClickListener(e -> {
            download(downloadTextOldStyle, false);
        });
        
        final Button downloadWindow = new Button("Download window");
        downloadWindow.addClickListener(e -> {
            DownloadTextWindow window = downloadTextWindow.get();
            window.init(downloadWindow.getCaption() + ".txt", TEXT_TO_DOWNLOAD);
        });

        buttonLayout.addComponents(downloadText, downloadTextOldStyle, downloadWindow);

        final Label textToDownload = new Label(TEXT_TO_DOWNLOAD);
        textToDownload.setWidth("66%");

        layout.addComponents(buttonLayout, textToDownload);
        layout.setMargin(true);
        layout.setSpacing(true);

        setContent(layout);
    }

    public void download(Button button, boolean v7Style) {
        StreamResource sr = getFileResource(button.getCaption() + ".txt");
        if (v7Style) {
            FileDownloader fd = new FileDownloader(sr);
            fd.extend(button);
        } else {
            Page.getCurrent().open(sr, "_self", false);
        }
    }
    
    public StreamResource getFileResource(String fileName) {
        StreamResource sr = new StreamResource(() -> new ByteArrayInputStream(TEXT_TO_DOWNLOAD.getBytes()), fileName);
        sr.setMIMEType("application/octet-stream");
        sr.setCacheTime(0);
        sr.getStream().setParameter("Content-Disposition", "attachment;filename=\"" + fileName + "\"");
        sr.getStream().setParameter("Content-Transfer-Encoding", "binary");
        return sr;
    }
}
